#include <string>
#include <qapplication.h>
#include <qlabel.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qpopupmenu.h>
#include <qtextstream.h>
#include <qlineedit.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qmessagebox.h>
#include <qlistview.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qpixmap.h>
#include <qwidget.h>
#include <qregexp.h>
#include <qframe.h>
#include <qmenubar.h>
#include <qradiobutton.h>
#include <qbutton.h>
#include <qbuttongroup.h>

static const char *newfolder_icon[] = {
/* width height num_colors chars_per_pixel */
"    21    21       47            1",
/* colors */
". c #ffffff",
"# c #080808",
"a c #292929",
"b c #636363",
"c c #6b6b6b",
"d c #737373",
"e c #7b7b7b",
"f c #848484",
"g c #8c8c8c",
"h c #949494",
"i c #9c9c9c",
"j c #a5a5a5",
"k c #adadad",
"l c #b5b5b5",
"m c #bdbdbd",
"n c #c6c6c6",
"o c #cecece",
"p c #d6d6d6",
"q c #e7e7e7",
"r c #efefef",
"s c #f7f7f7",
"t c #f7e7e7",
"u c #efdede",
"v c #f7efde",
"w c #ad9c5a",
"x c #847331",
"y c #b59c42",
"z c #8c845a",
"A c #a59442",
"B c #948439",
"C c #bdad4a",
"D c #c6b54a",
"E c #f7f7e7",
"F c #52524a",
"G c #313129",
"H c #181810",
"I c #ffff00",
"J c #424a39",
"K c #525a52",
"L c #5a635a",
"M c #4a524a",
"N c #424a42",
"O c #638463",
"P c #314239",
"Q c #525a5a",
"R c #00294a",
"S c #000000",
/* pixels */
"....S.I......s.......",
"....S.I......s.......",
"..ISMSI....I.........",
"..SINBISS.I.....s....",
"..SdIGIxxISMHSrrs....",
"..ScbIIIIJBAyOSSs....",
"IIIIIIIIIIIIICLLs....",
"..ScbIIIIdFPHQDD#....",
"..ScbIIIIdFPHQDD#....",
"..SdIcIddIijmhRRzE...",
"..SIefIgghImolaawv...",
"..IhhhIiijlISSSSSSSS.",
"..SijkIkkmpSrs.....SK",
"..SadkmnnpuSsSSSSS.SK",
"..ssSadllqtS.S.....SK",
"..ssSadllqtS.SSSS..SK",
"....s.SaaeoS.S.....SK",
".....s...SaS.S.....SK",
"...........S.S.....SK",
"...........SSSSSSSSSK",
"............KKKKKKKKK"
};

static const char *upwards_icon[] = {
 "21 21 26 1",
" 	c None",
".	c #000000",
"+	c #16140C",
"@	c #CEBF70",
"#	c #665E37",
"$	c #272415",
"%	c #71683D",
"&	c #91864F",
"*	c #7F7545",
"=	c #6D653B",
"-	c #242113",
";	c #5B5432",
">	c #FFEB8A",
",	c #FFEC8A",
"'	c #FEEB8A",
")	c #FFEC8B",
"!	c #544D2D",
"~	c #FFEB8B",
"{	c #FEEC8A",
"]	c #FEEC8B",
"^	c #57502F",
"/	c #F2E084",
"(	c #504A2B",
"_	c #040402",
":	c #0C0B06",
"<	c #040302",
"                     ",
"                     ",
"                     ",
"   .......           ",
"..+@@@@@@#.........  ",
".$%&&&&&&*=======-.  ",
".;>>>>>>>>,')))))!.  ",
".;>~~>.)~>'')))))!.  ",
".;)>>...>>>')).))!.  ",
".;>,.....>)')).))!.  ",
".;>{>~.'>),')).))!.  ",
".;),],.']),')).))!.  ",
".;)>>>.~>>>')).))!.  ",
".;)~>>.........))!.  ",
".;>')>~>))>')))))!.  ",
".^///////////////(.  ",
"._:::::::::::::::<.  ",
"                     ",
"                     ",
"                     ",
"                     "};

static const char* folder_icon[]={
    "16 16 9 1",
    "g c #808080",
    "b c #c0c000",
    "e c #c0c0c0",
    "# c #000000",
    "c c #ffff00",
    ". c None",
    "a c #585858",
    "f c #a0a0a4",
    "d c #ffffff",
    "..###...........",
    ".#abc##.........",
    ".#daabc#####....",
    ".#ddeaabbccc#...",
    ".#dedeeabbbba...",
    ".#edeeeeaaaab#..",
    ".#deeeeeeefe#ba.",
    ".#eeeeeeefef#ba.",
    ".#eeeeeefeff#ba.",
    ".#eeeeefefff#ba.",
    ".##geefeffff#ba.",
    "...##gefffff#ba.",
    ".....##fffff#ba.",
    ".......##fff#b##",
    ".........##f#b##",
    "...........####."};

static const char * locked_folder_xpm[] = {
/* width height num_colors chars_per_pixel */
"    18    18       71            2",
/* colors */
".. c #ffffff",
".# c #ffeab7",
".a c #ff0606",
".b c #ff0303",
".c c #ff0000",
".d c #fe2321",
".e c #fcfdfc",
".f c #fb0101",
".g c #faf4d9",
".h c #f8f1d3",
".i c #f6f6f6",
".j c #f5e9b7",
".k c #f1eddc",
".l c #efefef",
".m c #ecedec",
".n c #e0e0e0",
".o c #e0d5a5",
".p c #dfd5a8",
".q c #dad3b2",
".r c #d9cc95",
".s c #d5d2c2",
".t c #d4bf55",
".u c #d0d0d0",
".v c #cfcfcf",
".w c #c5c2b5",
".x c #bfbfbf",
".y c #b5b5b5",
".z c #b2b1ae",
".A c #b0b0b0",
".B c #aea064",
".C c #9f9fa0",
".D c #9f9f9f",
".E c #999686",
".F c #8b885e",
".G c #88867c",
".H c #808080",
".I c #776c3c",
".J c #727661",
".K c #706d5e",
".L c #6f9171",
".M c #606863",
".N c #565343",
".O c #465e51",
".P c #434343",
".Q c #404040",
".R c #333333",
".S c #313131",
".T c #2f2f2f",
".U c #2b2d2b",
".V c #262626",
".W c #252525",
".X c #1f1e15",
".Y c #1d1d1d",
".Z c #1d1b0f",
".0 c #1b1b1b",
".1 c #191919",
".2 c #171717",
".3 c #171616",
".4 c #131411",
".5 c #131313",
".6 c #101010",
".7 c #0e0e0e",
".8 c #0a0a0a",
".9 c #080808",
"#. c #080805",
"## c #070707",
"#a c #050505",
"#b c #030301",
"#c c #010101",
"#d c #002f4f",
"#e c #000000",
/* pixels */
"......#e............................",
"....#e.M#b..........................",
"..#e.Q.M.t.J#b......................",
"..#e.C.E.N.J.t.J#b.M.X#b............",
"..#e.C.C.C.E.N.J.t.t.t.L#b..........",
"..#e.C.C.C.A.A.K.L.t.t.t.M..........",
"..#e.C.C.A.A.w.V.B.O.X.M.t.6...e#e..",
"..#e.C.A.z.r#e.c#e.s.n.D#d.F.g#e.c##",
"..#e.A.A.w.S.c.c.c.1.j.x.I.Z.c.c.a##",
"..#e.A.x.x.q.2.c.c.c.3.o#c.c.c.c#e..",
"..#e.A.x.v.u.p.W.c.c.c#e.c.c.d#e.i..",
"..#e.T.H.x.u.n.k#a.c.c.c.c#e.h......",
"......#e.T.H.x.l.##e.c.c.c#e.h......",
"..........#e.T.G.P.c.c.c.c.c#..4....",
"...........i.m#e.c.c.c.2.c.c.c#e.U..",
"............#e.c.c.c#e.y.8.c.c.c.9..",
".............R.c.c.W..#e.Y.9.f.c.b#e",
"...............7.5..........#e#e.0.."
};

static const char * pix_file []={
    "16 16 7 1",
    "# c #000000",
    "b c #ffffff",
    "e c #000000",
    "d c #404000",
    "c c #c0c000",
    "a c #ffffc0",
    ". c None",
    "................",
    ".........#......",
    "......#.#a##....",
    ".....#b#bbba##..",
    "....#b#bbbabbb#.",
    "...#b#bba##bb#..",
    "..#b#abb#bb##...",
    ".#a#aab#bbbab##.",
    "#a#aaa#bcbbbbbb#",
    "#ccdc#bcbbcbbb#.",
    ".##c#bcbbcabb#..",
    "...#acbacbbbe...",
    "..#aaaacaba#....",
    "...##aaaaa#.....",
    ".....##aa#......",
    ".......##......."};

static const char * locked_file_xpm[] = {
"16 16 10 1",
" 	c None",
".	c #000000",
"+	c #FFFFFF",
"@	c #FF0015",
"#	c #808080",
"$	c #F95263",
"%	c #FFDCA8",
"&	c #FFFFC0",
"*	c #808000",
"=	c #C0C000",
"                ",
"                ",
"                ",
"        ..      ",
"       .++..    ",
"      .++@@#..  ",
"     .+$@@#+++. ",
"    .+$#$$++++. ",
"   .+$#@#%$++$. ",
"  .&&@$$%$++$.  ",
" .&&@@#+$&+$.   ",
" .$@@#&&&+$.    ",
"  .##$&&&$.     ",
"   ...**=.      ",
"      ...       ",
"                "};

static const char *right_arrow[] = {
/* width height num_colors chars_per_pixel */
"    44    29       16            1",
/* colors */
". c #000000",
"# c #a2a2a2",
"a c #595959",
"b c #717171",
"c c #616161",
"d c #aeaeae",
"e c #bababa",
"f c #9a9a9a",
"g c #202020",
"h c #303030",
"i c #080808",
"j c #101018",
"k c #8a8a8a",
"l c #797979",
"m c #494949",
"n c #383838",
/* pixels */
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeb#eeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeedgjaleeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeedh..inb#eeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeem.....gafeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeea.......imbdeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeajhhhmaaanimeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeefhdeeeefanc#eeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeefhdeefanc#eeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeekaekanc#eeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeebjmmb#eeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeebnbdeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeedeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee"
};

static const char* left_arrow[] = {
/* width height num_colors chars_per_pixel */
"    44    29       16            1",
/* colors */
". c #000000",
"# c #a2a2a2",
"a c #595959",
"b c #717171",
"c c #616161",
"d c #aeaeae",
"e c #bababa",
"f c #9a9a9a",
"g c #202020",
"h c #303030",
"i c #080808",
"j c #101018",
"k c #8a8a8a",
"l c #797979",
"m c #494949",
"n c #383838",
/* pixels */
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeee#beeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeelajgdeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeee#bni..hdeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeefag.....meeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeedbmi.......aeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeminaaamhhhjaeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeee#cnafeeeedhfeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeee#cnafeedhfeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeee#cnakeakeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeee#bmmjbeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeedbnbeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeedeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee"
};

class MyListView : public QListView
{
  Q_OBJECT
public:
    MyListView(QWidget* parent, const char* name, int, int, int, int);
    ~MyListView();
    enum FilterSpec { Dirs          = 0x001,
                      Files         = 0x002,
                      Drives        = 0x004,
                      NoSymLinks    = 0x008,
                      All           = 0x007,
                      TypeMask      = 0x00F };
    void dirList();
    QStringList getViewfile();
    MyListView(QWidget* parent, const char* name, QStringList, int);
    MyListView(const char* name, QString);
private slots:
    void slotChangeDir(QListViewItem*);
    void slotFilter(const QString&);
    void slotRightMenu(QListViewItem*, const QPoint &, int);
    void slotTestFile();
    void slotListClose();
    void slotUnpack();
    void slotFinallyUnpack();
    void slotCancel();
    void slotDirChange(const QString&);
    void slotcdup();
    void slotnewfolder();
    void slotCreatenew();
    void slotnewCancel();
    void slotMakeTar();
private:
    friend class QListView;
    QListView *navn;
    QString viewfile;
    QDir dir;
    QListViewItem* dirItem;
    QLineEdit *filter;
    QLabel* flabel;
    QListViewItem* tester;
    QString streng;
    QString locator;
    QFileInfo testobject;
    QStringList contents;
    QString files;
    QListBox* listing;
    QPushButton* button;
    QString bzipfile;
    char buf[100];
    QWidget* filelist;
    QString s;
    QFile temporary;
    QPushButton* okB;
    QMessageBox* warnings;
    QWidget* frame;
    QPushButton* buttoncreate;
    QPushButton* cancelB;
    QPushButton* newfolderbutton;
    QPushButton* upbutton;
    QComboBox* filename;
    QLabel* question;
    QLineEdit* savefile;
    QLabel* savelabel;
    QString dirlocation;
    QWidget* newfolder;
    QLineEdit* newfoldername;
    QPushButton* cancelnew;
    QStringList selectedfiles;
    int packercase;
    const char* pathn;
    const char* allfiles;
    const char* tarfile;
    const char* firstfile;
    const char* createlocation;
//    QString passthrough;
};

class MyWidget : public QWidget
{
 Q_OBJECT
public:
    MyWidget( QWidget *parent=0, const char *name=0 );
    ~MyWidget();
private slots:
    void slotAbout();
    void slotWhatisit();
    void slotLeft2Right();
    void slotRight2Left();
    void slotCreateTar();
    void slotUnpack();

private:
    friend class MyListView;
    MyListView* newlist;
    MyListView* another;
    QMenuBar* _menubar;
    QPopupMenu* _filemenu;
    QPopupMenu* _helpmenu;
    QPushButton* right;
    QPushButton* left;
    QListViewItem* tester;
    QListView* tarfiles;
    QDir dir;
    QString locator;
    QString files;
    QFileInfo testobject;
    QListViewItem* tarItem;
    MyListView* towhere;
    QWidget* frame;
    MyListView* utowhere;
    QMessageBox* warnings;
    int zipcase;
    QRadioButton* Bzip2;
    QRadioButton* Gzip;
    QRadioButton* Zip;
    QStringList selectedfiles;
};

#include "qutar.moc"

MyListView::MyListView(QWidget* parent, const char* name, int x, int y, int width, int length)
{
    navn = new QListView(parent, name);
    navn->addColumn(name);
    navn->setGeometry(x,y,width,length);
    navn->setMultiSelection(TRUE);
    filter = new QLineEdit(parent);
    filter->setGeometry(x + 35, y+length+5, width-50, 20);
    flabel = new QLabel("Filter", parent);
    flabel->setGeometry(x, y+length+5, 35, 20);
    connect(filter, SIGNAL(textChanged(const QString&)),this,SLOT(slotFilter(const QString&)) );
    dirList();
    connect(navn, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotChangeDir(QListViewItem*) ) );
    connect(navn, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint &, int)), this, SLOT(slotRightMenu(QListViewItem*, const QPoint &, int)));
}

MyListView::MyListView(QWidget* parent, const char* name, QStringList recieved, int zipcase)
{
    selectedfiles = recieved;
    packercase = zipcase;
    frame = new QWidget(0, "Select");
    frame->resize(310,500);
    frame->setCaption("Choose name....");
    locator = dir.absPath();

    buttoncreate = new QPushButton( "Create", frame, "create" );
    buttoncreate->setGeometry( 30, 450, 75, 30 );
    connect( buttoncreate, SIGNAL(clicked()), this, SLOT(slotMakeTar()) );

    cancelB = new QPushButton( "Cancel", frame, "cancel" );
    cancelB->setGeometry( 110, 450, 75, 30 );
    connect( cancelB, SIGNAL(clicked()), this, SLOT(slotCancel()) );

    upbutton = new QPushButton(frame);
    upbutton->setPixmap(upwards_icon);
    upbutton->setGeometry(250, 10, 23, 23);
    connect(upbutton, SIGNAL(clicked()), this, SLOT(slotcdup()));

    newfolderbutton = new QPushButton(frame);
    newfolderbutton->setPixmap(newfolder_icon);
    newfolderbutton->setGeometry(275, 10, 23, 23);
    connect(newfolderbutton, SIGNAL(clicked()), this, SLOT(slotnewfolder()));

    // Create an editable Combobox and a label below...
    filename = new QComboBox( TRUE, frame );
    filename->setGeometry(100, 350, 200, 30);

    question = new QLabel("Current location: ", frame);
    question->setGeometry(10, 350, 90, 30);
    dirlocation = dir.absPath();
    filename->insertItem( dirlocation, 0 );
    filename->show();
    connect(filename, SIGNAL(activated(const QString &)), this, SLOT(slotDirChange(const QString &)));

    savefile = new QLineEdit(frame);
    connect(savefile, SIGNAL(textChanged(const QString&)),this,SLOT(slotFilter(const QString&)) );
    savefile->setGeometry(100,380,200,30);
    savefile->setFocus();

    savelabel = new QLabel("Create file:", frame);
    savelabel->setText("Create file:");
    savelabel->setGeometry(10, 380, 90, 30);
    connect( savefile, SIGNAL( returnPressed() ), this, SLOT(slotMakeTar()));

    navn = new QListView(frame, "opentar");
    navn->addColumn("Select location");
    navn->setRootIsDecorated(FALSE);
    navn->setGeometry(10,40,288,300);
    dirList();
    connect(navn, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotChangeDir(QListViewItem*) ) );
    frame->show();
}

MyListView::MyListView(const char* name, QString temp)
{
    viewfile = temp;
    frame = new QWidget(0, "Chooser");
    frame->resize(300,500);
    frame->setCaption("Select location....");
    locator = dir.absPath();

    buttoncreate = new QPushButton( "Unpack", frame, "create" );
    buttoncreate->setGeometry( 30, 450, 75, 30 );
    connect( buttoncreate, SIGNAL(clicked()), this, SLOT(slotFinallyUnpack()) );

    cancelB = new QPushButton( "Cancel", frame, "cancel" );
    cancelB->setGeometry( 110, 450, 75, 30 );
    connect( cancelB, SIGNAL(clicked()), this, SLOT(slotCancel()) );

    upbutton = new QPushButton(frame);
    upbutton->setPixmap(upwards_icon);
    upbutton->setGeometry(250, 10, 23, 23);
    connect(upbutton, SIGNAL(clicked()), this, SLOT(slotcdup()));

    newfolderbutton = new QPushButton(frame);
    newfolderbutton->setPixmap(newfolder_icon);
    newfolderbutton->setGeometry(275, 10, 23, 23);
    connect(newfolderbutton, SIGNAL(clicked()), this, SLOT(slotnewfolder()));

    // Create an editable Combobox and a label below...
    filename = new QComboBox( TRUE, frame );
    filename->setGeometry(100, 350, 200, 30);

    question = new QLabel("Current location: ", frame);
    question->setGeometry(10, 350, 90, 30);
    dirlocation = dir.absPath();
    filename->insertItem( dirlocation, 0 );
    connect(filename, SIGNAL(activated(const QString &)), this, SLOT(slotDirChange(const QString &)));

    navn = new QListView(frame, "opentar");
    navn->addColumn("Select location");
    navn->setRootIsDecorated(FALSE);
    navn->setGeometry(10,40,288,300);
    dirList();
    connect(navn, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotChangeDir(QListViewItem*) ) );
    frame->show();
}

void MyListView::slotDirChange(const QString& combotext)
{
    navn->clear();
    testobject.refresh();
    testobject = combotext;
    testobject.setFile(dir.absFilePath(combotext));
    locator = dir.absPath();
    locator = locator+"/";
    if (testobject.isDir()) {
        dir.cd(dir.absFilePath(combotext));
        dirlocation = dir.absPath();
        filename->insertItem( dirlocation, 0 );
        dirList();
    }
}
void MyListView::slotChangeDir(QListViewItem* b)
{
    testobject.refresh();
    QString c = b->text(0);
    testobject = c;
    testobject.setFile(dir.absFilePath(c));
    locator = dir.absPath();
    if (testobject.isDir()) {
       if (testobject.isReadable() ) {
           navn->clear();
           dir.cd(dir.absFilePath(c));
           dirList();
       }
    else {

       warnings = new QMessageBox(this);
       warnings->warning(this, "I can't let you do that Dave.",
                               "It is locked\n"
			       "(indicated by the red x)");
    }
    }
}

void MyListView::slotCancel()
{
   frame->close();
}

// List the contents of the directory

void MyListView::dirList()
{
//First, we list all the directories, pattern is turned off
      dir.setNameFilter("*");
      int it=0;
      while ( dir[it] != 0 ) {
        files = dir[it];
	testobject = files;
        testobject.setFile(dir.absFilePath(files));
       if ( testobject.isReadable() ) {
        if ( testobject.isDir()) {
	        dirItem = new QListViewItem(navn, files);
		dirItem->setPixmap(0, folder_icon);
                navn->insertItem(dirItem);
        }
       }
        else {
          if ( testobject.isDir()) {
	        dirItem = new QListViewItem(navn, files);
		dirItem->setPixmap(0, locked_folder_xpm);
                navn->insertItem(dirItem);
        }
      }
      ++it;
     }

//Second, we turn on the filter, and let the user see only the selected file pattern
     dir.setNameFilter(streng);
     it=0;
      while ( dir[it] != 0 ) {
        files = dir[it];
	testobject = files;
	testobject.setFile(dir.absFilePath(files));
       if ( testobject.isReadable() ) {
        if ( testobject.isFile()) {
	        dirItem = new QListViewItem(navn, files);
		dirItem->setPixmap(0, pix_file);
                navn->insertItem(dirItem);
        }
       }
        else {
         if ( testobject.isFile()) {
	        dirItem = new QListViewItem(navn, files);
		dirItem->setPixmap(0, locked_file_xpm);
                navn->insertItem(dirItem);
         }
       }
      ++it;
     }
}

void MyListView::slotMakeTar()
{
    frame->close();
    QStringList::Iterator it = ++selectedfiles.begin();
	files = *it;
        ++it;
        dirlocation = dir.absPath();
        QString first = files;
        QString filepath = dir.absFilePath(first);
        int sy = filepath.length();
        int location = filepath.findRev(QRegExp("/"), -1);
        int flength = sy - location;
        QString temp = filepath;
        QString pathname = temp.remove(location+1, sy);
        QString fname = filepath.remove(0, location+1);
        createlocation = dirlocation;
        pathn = pathname.latin1();
        tarfile = streng.latin1();
        firstfile = fname.latin1();
        sprintf(buf, "tar cf \"%s\"/\"%s\".tar -C \"%s\" \"%s\"", createlocation, tarfile, pathn, firstfile);	
        system(buf);
   for ( it = it; it != selectedfiles.end(); ++it) {
	files = *it;
        filepath = dir.absFilePath(files);
        sy = filepath.length();
        location = filepath.findRev(QRegExp("/"), -1);
        flength = sy - location;
        temp = filepath;
        pathname = temp.remove(location+1, sy);
        fname = filepath.remove(0, location+1);
        pathn = pathname.latin1();
	allfiles = fname.latin1();
 	sprintf(buf, "tar rf \"%s\"/\"%s\".tar -C \"%s\" \"%s\"", createlocation, tarfile, pathn, allfiles);  	
        system(buf);
    }	

	if (packercase == 1 ) {
		sprintf(buf, "bzip2 \"%s\"/\"%s\".tar", createlocation, tarfile);		
		system(buf);
	}
	if (packercase == 2) {
		sprintf(buf, "gzip \"%s\"/\"%s\".tar", createlocation, tarfile);
		system(buf);
	}
	if (packercase == 3) {
		sprintf(buf, "zip \"%s\"/\"%s\".zip \"%s\".tar", createlocation, tarfile, tarfile);
		system(buf);

		sprintf(buf, "rm -f \"%s\"/\"%s\".tar", createlocation, tarfile);
		system(buf);
        }
}

void MyListView::slotRightMenu(QListViewItem* b, const QPoint &pos, int loc)
 {
//    passthrough = b->text(0);
    viewfile = b->text(0);
    bzipfile = viewfile;
    locator = dir.absPath();
    locator = locator+"/";
    viewfile = locator + viewfile;
    navn->setSelected(b, TRUE);
    QPopupMenu *file = new QPopupMenu( this );
    CHECK_PTR( file );
    file->insertItem("Extract to", this, SLOT(slotUnpack()));
    file->insertItem("View contents", this, SLOT(slotTestFile()));
    file->popup(pos,loc);
}

void MyListView::slotcdup()
{
    dir.cdUp();
    navn->clear();
    dirList();
}

void MyListView::slotnewfolder()
{
   streng = ""; //clear a buffer for filename
   newfolder = new QWidget(this, "New folder name....", WStyle_NormalBorder | WStyle_Title | WType_Modal);
   newfolder->resize(200, 150);
   QLabel* info = new QLabel("What is the new folder", newfolder);
   QLabel* info2 = new QLabel("going to be called?", newfolder);
   info->setGeometry(10,1,150,20);
   info2->setGeometry(10,21,150,20);

   newfoldername = new QLineEdit(newfolder);
   connect(newfoldername, SIGNAL(textChanged(const QString&)),this,SLOT(slotFilter(const QString&)) );
   newfoldername->setGeometry(10,50,150,30);
   newfoldername->setFocus();
   connect( newfoldername, SIGNAL( returnPressed() ), this, SLOT(slotCreatenew()));

   cancelnew = new QPushButton( "Cancel", newfolder, "cancel" );
   cancelnew->setGeometry( 10, 100, 75, 30 );
   connect( cancelnew, SIGNAL(clicked()), this, SLOT(slotnewCancel()) );

   QPushButton* okay = new QPushButton("Ok", newfolder, "ok");
   okay->setGeometry( 90, 100, 75, 30 );
   connect(okay, SIGNAL(clicked()), this, SLOT(slotCreatenew()));
   newfolder->show();
}

void MyListView::slotFinallyUnpack()
{
    int n;
    QString dirlocation = dir.absPath();
    filelist = new QWidget( 0, "Upack");
    filelist->resize(400,400);
    okB = new QPushButton("Ok", filelist);
    okB->setGeometry(165, 365, 70,30);
    connect(okB, SIGNAL(clicked()), this, SLOT(slotListClose()));
    listing = new QListBox(filelist);
    listing->setGeometry(10, 10, 350,350);
    okB = new QPushButton("Ok", filelist);
    okB->setGeometry(165, 365, 70,30);
    connect(okB, SIGNAL(clicked()), this, SLOT(slotListClose()));
    const char* what = "0";
    const char* locations;
    what = viewfile;
    sprintf(buf, "file \"%s\" > temp", what);
    system(buf);
    temporary.setName("temp");
    temporary.open(IO_ReadOnly);
    QTextStream t( &temporary );
        s = t.readLine();
        if (s.contains("gzip compressed data", TRUE) || s.contains("compress'd data 16 bits", TRUE)) {
		temporary.close();
		sprintf(buf, "rm -f temp");
    		system(buf);
		sprintf(buf, "gunzip -l \"%s\" > temp", what);
		system(buf);
                if ( temporary.open(IO_ReadOnly) ) {
                    QTextStream t( &temporary );
                    n = 1;
                      while ( !t.eof() ) {
                       s = t.readLine();
                      }
                }
                temporary.close();
		sprintf(buf, "rm -f temp");
                system(buf);
                if (s.contains(".tar", TRUE)) {
                  locations = dirlocation;
                  sprintf(buf, "tar zxvf \"%s\" -C \"%s\" > temp", what, locations);
                  system(buf);
                  if ( temporary.open(IO_ReadOnly) ) {
                    QTextStream t( &temporary );
                    n = 1;
                    while ( !t.eof() ) {
                      s = t.readLine();
                      listing->insertItem(s, n);
	              n++;
                    }
                    temporary.close();
                    sprintf(buf, "rm -f temp");
                    system(buf);
                  }
                }
                else {
                    listing->insertItem(s, n);
			sprintf(buf, "gunzip \"%s\"", what);
			system(buf);
                }
        }
        else
        if (s.contains("GNU tar archive", TRUE)) {
		temporary.close();
		sprintf(buf, "rm -f temp");
                system(buf);
		sprintf(buf, "tar xvf \"%s\" -C \"%s\"> temp", what, locations);
                  system(buf);
                  if ( temporary.open(IO_ReadOnly) ) {
                    QTextStream t( &temporary );
                    n = 1;
                    while ( !t.eof() ) {
                      s = t.readLine();
                      listing->insertItem(s, n);
	              n++;
                    }
                    temporary.close();
                    sprintf(buf, "rm -f temp");
                    system(buf);
                  }
        }
        else
        if (s.contains("Zip archive data", TRUE)) {
            int cont = 0;
	    temporary.close();
            sprintf(buf, "rm -f temp");
            system(buf);
            sprintf(buf, "unzip -l \"%s\" > temp", what);
	    system(buf);
                if ( temporary.open(IO_ReadOnly) ) {
                    QTextStream t( &temporary );
                    n = 1;
                      while ( !t.eof() ) {
                       s = t.readLine();
                       listing->insertItem(s, n);
                       n++;
                         if (s.contains( ".tar", FALSE)) {
                                cont = 1;
                         }
                      }
                }
		temporary.close();
              if (cont == 1) {
		     sprintf(buf, "rm -f temp");
                     system(buf);
                     sprintf(buf, "tar xvf \"%s\" -C \"%s\"> temp", what, locations);
                     system(buf);
                     if ( temporary.open(IO_ReadOnly) ) {
                        QTextStream t( &temporary );
                        n = 1;
                        while ( !t.eof() ) {
                          s = t.readLine();
                          listing->insertItem(s, n);
	                  n++;
                        }
                     }
              }
              else {
//                 listing->insertItem(s, n);
		locations = dirlocation;
		sprintf(buf, "unzip  \"%s\" -d \"%s\" > temp", what, locations);
		system(buf);
                if ( temporary.open(IO_ReadOnly) ) {
                   QTextStream t( &temporary );
                      n = 1;
                      while ( !t.eof() ) {
                        s = t.readLine();
                        listing->insertItem(s, n);
	                n++;
                      }
                      temporary.close();
                      sprintf(buf, "rm -f temp");
                      system(buf);
                }
              }
	temporary.close();
	sprintf(buf, "rm -f temp");
    	system(buf);
        }
        else
        if (s.contains( "bzip2 compressed data", TRUE)) {
                locations = dirlocation;
		temporary.close();
		sprintf(buf, "rm -f temp");
    		system(buf);
                what = viewfile;
                sprintf(buf, "mv \"%s\" \"%s\"", what, locations);
		system(buf);
                bzipfile = dirlocation + "/" + bzipfile;
                const char* bunzipfile;
		bunzipfile = bzipfile;
                sprintf(buf, "bunzip2 \"%s\"", bunzipfile);
		system(buf);
//		towhere->close();
                what = "";
        }
        else {
             s = "I do not find this to be an archive";
             listing->insertItem(s, n);
	     s = "maybe I am wrong, maybe you are wrong";
             listing->insertItem(s, n+1);
        }
    temporary.close();
    frame->close();
    filelist->show();
}

//This is the slot where we just let the user see the contents of the file

void MyListView::slotTestFile()
{
    int n;
    filelist = new QWidget( 0, "Testing");
    filelist->resize(400,400);
    listing = new QListBox(filelist);
    listing->setGeometry(10, 10, 350,350);
    okB = new QPushButton("Ok", filelist);
    okB->setGeometry(165, 365, 70,30);
    connect(okB, SIGNAL(clicked()), this, SLOT(slotListClose()));
    const char* what = "0";
    what = viewfile;
    sprintf(buf, "file \"%s\" > temp", what);
    system(buf);
    temporary.setName("temp");
    temporary.open(IO_ReadOnly);
    QTextStream t( &temporary );
        s = t.readLine();
        if (s.contains("gzip compressed data", TRUE) || s.contains("compress'd data 16 bits", TRUE)) {
		temporary.close();
		sprintf(buf, "rm -f temp");
    		system(buf);
		sprintf(buf, "gunzip -l \"%s\" > temp", what);
		system(buf);
                if ( temporary.open(IO_ReadOnly) ) {
                    QTextStream t( &temporary );
                    n = 1;
                      while ( !t.eof() ) {
                       s = t.readLine();
                      }
                }
                temporary.close();
		sprintf(buf, "rm -f temp");
                system(buf);
                if (s.contains(".tar", TRUE)) {
                  sprintf(buf, "tar ztvf \"%s\" > temp", what);
                  system(buf);
                  if ( temporary.open(IO_ReadOnly) ) {
                    QTextStream t( &temporary );
                    n = 1;
                    while ( !t.eof() ) {
                      s = t.readLine();
                      listing->insertItem(s, n);
	              n++;
                    }
                    temporary.close();
                    sprintf(buf, "rm -f temp");
                    system(buf);
                  }
                }
                else {
                    listing->insertItem(s, n);
                }
        }
        else
        if (s.contains("GNU tar archive", TRUE)) {
		temporary.close();
		sprintf(buf, "rm -f temp");
                system(buf);
		sprintf(buf, "tar tvf \"%s\" > temp", what);
                  system(buf);
                  if ( temporary.open(IO_ReadOnly) ) {
                    QTextStream t( &temporary );
                    n = 1;
                    while ( !t.eof() ) {
                      s = t.readLine();
                      listing->insertItem(s, n);
	              n++;
                    }
                    temporary.close();
                    sprintf(buf, "rm -f temp");
                    system(buf);
                  }
        }
        else
        if (s.contains("Zip archive data", TRUE)) {
            int cont = 0;
	    temporary.close();
            sprintf(buf, "rm -f temp");
            system(buf);
            sprintf(buf, "unzip -l \"%s\" > temp", what);
	    system(buf);
                if ( temporary.open(IO_ReadOnly) ) {
                    QTextStream t( &temporary );
                    n = 1;
                      while ( !t.eof() ) {
                       s = t.readLine();
                       listing->insertItem(s, n);
                       n++;
                         if (s.contains( ".tar", FALSE)) {
                                cont = 1;
                         }
                      }
                }
		temporary.close();
              if (cont == 1) {
		     sprintf(buf, "rm -f temp");
                     system(buf);
                     sprintf(buf, "tar ztvf \"%s\" > temp", what);
                     system(buf);
                     if ( temporary.open(IO_ReadOnly) ) {
                        QTextStream t( &temporary );
                        n = 1;
                        while ( !t.eof() ) {
                          s = t.readLine();
                          listing->insertItem(s, n);
	                  n++;
                        }
                     }
              }
              else {
                    listing->insertItem(s, n);
              }
	temporary.close();
	sprintf(buf, "rm -f temp");
    	system(buf);
        }
        else
        if (s.contains( "bzip2 compressed data", TRUE)) {
             s = "Ok, this is a bzip2 file.";
	     listing->insertItem(s,n);
             s = "But I cannot tell you what is in it";
             listing->insertItem(s,n+1);
	     s = "because bzip2 does not have a tell function.";
             listing->insertItem(s,n+1);
		temporary.close();
		sprintf(buf, "rm -f temp");
    		system(buf);
        }
        else {
             s = "I do not find this to be an archive";
             listing->insertItem(s, n);
	     s = "maybe I am wrong, maybe you are wrong";
             listing->insertItem(s, n+1);
        }
    temporary.close();
    filelist->show();
}

void MyListView::slotListClose()
{
   filelist->close();
}

QStringList MyListView::getViewfile()
{
    contents = "";
    int checksum = navn->childCount();
    tester = navn->firstChild();

    // Go through all items of liste
    int x = 0;
    while ( x < checksum ) {
        if (tester->isSelected()) {
		locator = dir.absPath();
		locator = locator+"/";
		files = tester->text(0);
		testobject = dir.absFilePath(files);
                  if (testobject.isReadable() ) {
		     if ( testobject.isDir()) {
                        files = locator+files;
                        contents.append(files);
                     }
	             if ( testobject.isFile()) {
                         files = locator+files;
                         contents.append(files);
                     }
                  }
        }
        tester = tester->nextSibling();
        x++;
    }	
  navn->clearSelection();
  return contents;
}

//Here we actually create the new folder

void MyListView::slotCreatenew()
{
   const char* newdirname;
   newdirname = streng;
   locator = dir.absPath();
   const char* p;
   p = locator;
   sprintf(buf, "mkdir \"%s\"/\"%s\"", p, newdirname);
   system(buf);
   newfolder->close();
   navn->clear();
   dirList();
}

void MyListView::slotnewCancel()
{
    newfolder->close();
}

void MyListView::slotFilter( const QString& newText )
{
    QString str("");
    str += newText;
    streng = str;
    navn->clear();
    dirList();
}

void MyListView::slotUnpack()
{
   MyListView* buggre = new MyListView("Hello", viewfile);
}

MyListView::~MyListView()
{
}

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
    setMinimumSize( 500, 500 );
    setMaximumSize(500,500);

    another = new MyListView(this, "Files", 10,40,200,300);

// Set up the tar files selection box
    tarfiles = new QListView( this );
    tarfiles->addColumn("Selected");
    tarfiles->setGeometry(280, 40, 200, 300);
    tarfiles->setMultiSelection(TRUE);
    tarfiles->show();
//    connect(tarfiles, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotdoubleRight2Left(QListViewItem*) ) );


    QPushButton *create = new QPushButton( "Create", this, "create" );
    create->setGeometry( 350, 400, 75, 30 );
    connect( create, SIGNAL(clicked()), this, SLOT(slotCreateTar()) );


// Set up the Create archive button so that we can do something useful
    QPushButton *untar = new QPushButton( "Unpack", this, "untar" );
    untar->setGeometry( 60, 400, 75, 30 );
    connect( untar, SIGNAL(clicked()), this, SLOT(slotUnpack()) );

// Set up and display the menu
    _helpmenu = new QPopupMenu;
    _helpmenu->insertItem("What is it?", this, SLOT(slotWhatisit()));
    _helpmenu->insertItem("&About Qutar", this, SLOT(slotAbout()));
    _filemenu = new QPopupMenu;
    _filemenu->insertItem("Quit", qApp, SLOT(quit()));
    _menubar = new QMenuBar(this);
    _menubar->insertItem("File", _filemenu);
    _menubar->insertSeparator();
    _menubar->insertItem("&Help", _helpmenu);

// All right, lets get the selector buttons up and running
    right = new QPushButton(this );
    right->setPixmap(right_arrow);
    left = new QPushButton(this);
    left->setPixmap(left_arrow);
    right->setGeometry(225, 170, 44, 29);
    left->setGeometry(225, 199, 44, 29);

// Connect them to the appropriate slots
    connect( right, SIGNAL( clicked() ), this, SLOT(slotLeft2Right() ));
    connect( left, SIGNAL( clicked() ), this, SLOT(slotRight2Left()));

// Next, let the user choose the compression format
// First we set up a group
    QButtonGroup *grp1 = new QButtonGroup( 1, QGroupBox::Horizontal, "Compression format", this);
    grp1->setGeometry(180,370,130,100);

// Then we insert the three buttons for the different formats
    Bzip2 = new QRadioButton( "&Bzip2", grp1 );
    Gzip = new QRadioButton( "&Gzip", grp1 );
    Zip = new QRadioButton( "&Zip", grp1 );

}

// Just prints out the "About" menu
void MyWidget::slotAbout()
{
	QMessageBox::information(this, "About Qutar",
					"v0.99_2 QPL license.\n"
					"Tars and untars files.\n"
                                        "Author: Geir Erik Nielsen\n"
					"e-mail:gen@nextel.no");
}

// Just prints out the help menu "What is it"
void MyWidget::slotWhatisit()
{
	QMessageBox::information(this, "What is it?",
					"It tars and untars files,\n"
					"gzips, bzip2s or zips them,\n"
					"in a nice graphical enviroment.\n");
}

void MyWidget::slotLeft2Right()
{
   QStringList dircontents = another->getViewfile();
   for ( QStringList::Iterator it = ++dircontents.begin(); it != dircontents.end(); ++it ) {
                locator = dir.absPath();
		locator = locator+"/";
		files = *it;
		testobject = dir.absFilePath(files);
		     if ( testobject.isDir()) {
		        tarItem = new QListViewItem(tarfiles, files);
		        tarItem->setPixmap(0, folder_icon);
		        tarfiles->insertItem(tarItem);
                     }
	             if ( testobject.isFile()) {
	                tarItem = new QListViewItem(tarfiles, files);
		        tarItem->setPixmap(0, pix_file);
		        tarfiles->insertItem(tarItem);
                     }
        }
}

void MyWidget::slotRight2Left()
{
    int checksum = tarfiles->childCount();
    tester = tarfiles->firstChild();

    // Go through all items of liste
    int x = 0;
    while ( x < checksum ) {
        if (tester->isSelected())
		tarfiles->removeItem(tester);
        tester = tester->nextSibling();
        x++;
    }	
}

void MyWidget::slotCreateTar()
{
    selectedfiles = "";
    int checksum = tarfiles->childCount();
    tester = tarfiles->firstChild();

    // Go through all items of liste
    int x = 0;
    int checker = tarfiles->childCount();
    if (checker != 0 ){
     while ( x < checksum ) {
        files = tester->text(0);
        tester = tester->nextSibling();
        x++;
        selectedfiles.append(files);
     }
     if (Bzip2->isChecked()) {
        zipcase = 1;
     }

     if (Gzip->isChecked()) {
        zipcase = 2;
     }

     if (Zip->isChecked()) {
        zipcase = 3;
     }

     towhere = new MyListView(this, "Save", selectedfiles, zipcase);
    }
    else {
      warnings = new QMessageBox(this);
      warnings->warning(this, "I can't let you do that Dave.",
                               "You have not yet selected\n"
			       "any files. First select files!");
    }
}

void MyWidget::slotUnpack()
{
   QString filethingy;
   QStringList dircontents = another->getViewfile();
   for ( QStringList::Iterator it = ++dircontents.begin(); it != dircontents.end(); ++it ) {
                locator = dir.absPath();
		locator = locator+"/";
		files = *it;
		testobject = dir.absFilePath(files);
	             if ( testobject.isFile()) {
	                filethingy = files;
                     }
   }
   utowhere = new MyListView("Save", filethingy);
}

MyWidget::~MyWidget()
{
   delete another;
}

int main( int argc, char **argv )
{
    QApplication a( argc, argv );

    MyWidget w;
    w.setGeometry( 100, 100, 500, 170 );
    a.setMainWidget( &w );
    w.show();
    return a.exec();
}